import { world, system, ItemStack, GameMode, EquipmentSlot, BlockVolume, BlockTypes } from '@minecraft/server';
import { unbreakable } from 'common/configs/unbreakable.js';
import { replaceable } from 'common/configs/replaceable.js';
import { damageItem } from 'common/durability_update.js';

world.afterEvents.playerInteractWithEntity.subscribe(e =>{
	let item = e.itemStack;
	let player = e.player;
	let target = e.target;
	system.run(() =>{
		confirmBuild(target, item, player);
		confirmDestroy(target, item, player);
	});
});

function confirmBuild(t, i, p){
	if(i == undefined) return;
	if(i.getTags().includes('sledgehammer')) return;
	if(t.typeId != 'raigen:blueprints_hologram') return;
	if(t.getComponent('minecraft:variant').value != 1) return;
	let isCreative = p.getGameMode() == 'creative' ? true : false;
	let fromCoords = t.getDynamicProperty('raigen:from_coords');
	let toCoords = t.getDynamicProperty('raigen:to_coords');
	let volume = new BlockVolume(fromCoords, toCoords);
	let amount = 0;
	try{
		let blocks = t.dimension.getBlocks(volume, {}, false);
		for(const loc of blocks.getBlockLocationIterator()){
			if(replaceable.includes(p.dimension.getBlock(loc).typeId)){
				amount++;
			}
		}
		if(isCreative){
			try{
				t.dimension.fillBlocks(blocks, i.typeId);
			}
			catch{
				throwErr(p, `§cThat item isn't valid`);
			}
		}
		else{
			if(BlockTypes.get(i.typeId) != undefined){
				let test = p.runCommand(`testfor @s[hasitem=[{item=${i.typeId}, quantity=${amount}..}]]`);
				if(test.successCount == 1){
					p.runCommand(`clear @s[m=!c] ${i.typeId} 0 ${amount}`);
					for(const loc of blocks.getBlockLocationIterator()){
						if(replaceable.includes(p.dimension.getBlock(loc).typeId)){
							p.runCommand(`setblock ${loc.x} ${loc.y} ${loc.z} ${i.typeId} destroy`);
						}
					}
				}
				else{
					throwErr(p, `§cYou have insufficient resources. You need ${amount}!`);
				}
			}
			else{
				throwErr(p, `§cThat item isn't valid`);
			}
		}
	}
	catch{
		throwErr(p, `§cBlocks within the specified coordinates aren't within render distance`);
	}
}

function confirmDestroy(t, i, p){
	if(i == undefined) return;
	if(t.typeId != 'raigen:blueprints_hologram' || !i.getTags().includes('sledgehammer')) return;
	let isCreative = p.getGameMode() == 'creative' ? true : false;
	let fromCoords = t.getDynamicProperty('raigen:from_coords');
	let toCoords = t.getDynamicProperty('raigen:to_coords');
	let volume = new BlockVolume(fromCoords, toCoords);
	let amount = 0;
	try{
		let blocks = t.dimension.getBlocks(volume, {}, false);
		for(const loc of blocks.getBlockLocationIterator()){
			if(!unbreakable.includes(p.dimension.getBlock(loc).typeId)){
				amount++;
			}
		}
		let durability = i.getComponent('durability');
		if(!isCreative){
			let it = 0;
			for(const loc of blocks.getBlockLocationIterator()){
				if(!unbreakable.includes(p.dimension.getBlock(loc).typeId)){
					it++;
					damageItem(p, 1, i);
					if(durability.damage < durability.maxDurability){
						p.runCommand(`setblock ${loc.x} ${loc.y} ${loc.z} air destroy`);
					}
				}
			}
		}
		else{
			t.dimension.fillBlocks(blocks, 'minecraft:air');
		}
	}
	catch{
		throwErr(p, `§cBlocks within the specified coordinates aren't within render distance`);
	}
}

function throwErr(p, e){
	p.runCommand(`title @s actionbar ${e}`);
	p.playSound('note.banjo', {location: p.location, pitch: 0.7, volume: 1});
}